@@ -19,6 +19,13 @@ interface UploadedPart {
   size: number;
   etag: string;
 }
+interface CompletionResponse {
+  complete_url: string;
+  body: string;
+}
+interface FinalizationResponse {
+  field_value: string;
+}
 
 export enum S3FileFieldResultState {
   Aborted,
@@ -48,33 +55,26 @@ export type S3FileFieldProgressCallback = (progress: S3FileFieldProgress) => voi
 
 export interface S3FileFieldClientOptions {
   readonly baseUrl: string;
-  readonly onProgress?: S3FileFieldProgressCallback;
   readonly apiConfig?: AxiosRequestConfig
 }
 
 export default class S3FileFieldClient {
   protected readonly api: AxiosInstance;
 
-  protected readonly onProgress: S3FileFieldProgressCallback;
-
   /**
    * Create an S3FileFieldClient instance.
    *
    * @param options {S3FileFieldClientOptions} - A Object with all arguments.
    * @param options.baseUrl - The absolute URL to the Django server.
-   * @param [options.onProgress] - A callback for upload progress.
    * @param [options.apiConfig] - An axios configuration to use for Django API requests.
    *                              Can be extracted from an existing axios instance via `.defaults`.
    */
   constructor(
     {
       baseUrl,
-      onProgress = () => { /* no-op */ },
       apiConfig = {},
     }: S3FileFieldClientOptions,
   ) {
-    this.onProgress = onProgress;
-
     this.api = axios.create({
       ...apiConfig,
       // Add a trailing slash
@@ -85,11 +85,11 @@ export default class S3FileFieldClient {
   /**
    * Initializes an upload.
    *
-   * @param file The file to upload.
-   * @param fieldId The Django field identifier.
+   * @param file - The file to upload.
+   * @param fieldId - The Django field identifier.
    */
   protected async initializeUpload(file: File, fieldId: string): Promise<MultipartInfo> {
-    const response = await this.api.post('upload-initialize/', {
+    const response = await this.api.post<MultipartInfo>('upload-initialize/', {
       field_id: fieldId,
       file_name: file.name,
       file_size: file.size,
@@ -100,10 +100,15 @@ export default class S3FileFieldClient {
   /**
    * Uploads all the parts in a file directly to an object store in serial.
    *
-   * @param file The file to upload.
-   * @param parts The list of parts describing how to break up the file.
+   * @param file - The file to upload.
+   * @param parts - The list of parts describing how to break up the file.
+   * @param onProgress - A callback for upload progress.
    */
-  protected async uploadParts(file: File, parts: PartInfo[]): Promise<UploadedPart[]> {
+  protected async uploadParts( // eslint-disable-line class-methods-use-this
+    file: File,
+    parts: PartInfo[],
+    onProgress: S3FileFieldProgressCallback,
+  ): Promise<UploadedPart[]> {
     const uploadedParts: UploadedPart[] = [];
     let fileOffset = 0;
     for (const part of parts) {
@@ -112,7 +117,7 @@ export default class S3FileFieldClient {
       const response = await axios.put(part.upload_url, chunk, {
         // eslint-disable-next-line @typescript-eslint/no-loop-func
         onUploadProgress: (e) => {
-          this.onProgress({
+          onProgress({
             uploaded: fileOffset + e.loaded,
             total: file.size,
             state: S3FileFieldProgressState.Sending,
@@ -134,13 +139,14 @@ export default class S3FileFieldClient {
    *
    * The object will exist in the object store after completion.
    *
-   * @param multipartInfo The information describing the multipart upload.
-   * @param parts The parts that were uploaded.
+   * @param multipartInfo - The information describing the multipart upload.
+   * @param parts - The parts that were uploaded.
    */
   protected async completeUpload(
-    multipartInfo: MultipartInfo, parts: UploadedPart[],
+    multipartInfo: MultipartInfo,
+    parts: UploadedPart[],
   ): Promise<void> {
-    const response = await this.api.post('upload-complete/', {
+    const response = await this.api.post<CompletionResponse>('upload-complete/', {
       upload_signature: multipartInfo.upload_signature,
       upload_id: multipartInfo.upload_id,
       parts,
@@ -155,7 +161,10 @@ export default class S3FileFieldClient {
         // to include in the signature validation, causing it to fail.
         // So, do not send this request with any Content-Type, as that is what's specified by the
         // CompleteMultipartUpload docs.
-        'Content-Type': null,
+        // Unsetting default headers via "transformRequest" is awkward (since the headers aren't
+        // flattened), so this is actually; the most straightforward way; the null value is passed
+        // through to XMLHttpRequest, then ignored.
+        'Content-Type': null as unknown as string,
       },
     });
   }
@@ -165,10 +174,10 @@ export default class S3FileFieldClient {
    *
    * This will only succeed if the object is already present in the object store.
    *
-   * @param multipartInfo Signed information returned from /upload-complete/.
+   * @param multipartInfo - Signed information returned from /upload-complete/.
    */
   protected async finalize(multipartInfo: MultipartInfo): Promise<string> {
-    const response = await this.api.post('finalize/', {
+    const response = await this.api.post<FinalizationResponse>('finalize/', {
       upload_signature: multipartInfo.upload_signature,
     });
     return response.data.field_value;
@@ -177,18 +186,23 @@ export default class S3FileFieldClient {
   /**
    * Uploads a file using multipart upload.
    *
-   * @param file The file to upload.
-   * @param fieldId The Django field identifier.
+   * @param file - The file to upload.
+   * @param fieldId - The Django field identifier.
+   * @param [onProgress] - A callback for upload progress.
    */
-  public async uploadFile(file: File, fieldId: string): Promise<S3FileFieldResult> {
-    this.onProgress({ state: S3FileFieldProgressState.Initializing });
+  public async uploadFile(
+    file: File,
+    fieldId: string,
+    onProgress: S3FileFieldProgressCallback = () => { /* no-op */ },
+  ): Promise<S3FileFieldResult> {
+    onProgress({ state: S3FileFieldProgressState.Initializing });
     const multipartInfo = await this.initializeUpload(file, fieldId);
-    this.onProgress({ state: S3FileFieldProgressState.Sending, uploaded: 0, total: file.size });
-    const parts = await this.uploadParts(file, multipartInfo.parts);
-    this.onProgress({ state: S3FileFieldProgressState.Finalizing });
+    onProgress({ state: S3FileFieldProgressState.Sending, uploaded: 0, total: file.size });
+    const parts = await this.uploadParts(file, multipartInfo.parts, onProgress);
+    onProgress({ state: S3FileFieldProgressState.Finalizing });
     await this.completeUpload(multipartInfo, parts);
     const value = await this.finalize(multipartInfo);
-    this.onProgress({ state: S3FileFieldProgressState.Done });
+    onProgress({ state: S3FileFieldProgressState.Done });
     return {
       value,
       state: S3FileFieldResultState.Successful,
