@@ -1,4 +1,4 @@
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional, cast
 
 from botocore.exceptions import ClientError
 from storages.backends.s3boto3 import S3Boto3Storage
@@ -7,22 +7,28 @@ if TYPE_CHECKING:
     # mypy_boto3_s3 only provides types
     import mypy_boto3_s3 as s3
 
-from ._multipart import MultipartManager, ObjectNotFoundException, TransferredParts
+from ._multipart import MultipartManager, ObjectNotFoundError, TransferredParts
 
 
 class Boto3MultipartManager(MultipartManager):
     def __init__(self, storage: 'S3Boto3Storage'):
         resource: s3.ServiceResource = storage.connection
-        self._client: s3.Client = resource.meta.client
+        self._client: s3.Client = cast('s3.Client', resource.meta.client)
         self._bucket_name: str = storage.bucket_name
 
-    def _create_upload_id(self, object_key: str) -> str:
+    def _create_upload_id(
+        self,
+        object_key: str,
+        content_type: Optional[str] = None,
+    ) -> str:
+        boto3_kwargs = {}
+        if content_type is not None:
+            boto3_kwargs['ContentType'] = content_type
         resp = self._client.create_multipart_upload(
             Bucket=self._bucket_name,
             Key=object_key,
+            **boto3_kwargs,  # type: ignore[arg-type]
             # TODO: filename in Metadata
-            # TODO: ContentType
-            # TODO: ContentEncoding
             # TODO: ensure ServerSideEncryption is set, even if not specified
             # TODO: use client._get_write_parameters?
         )
@@ -69,4 +75,4 @@ class Boto3MultipartManager(MultipartManager):
             )
             return stats['ContentLength']
         except ClientError:
-            raise ObjectNotFoundException()
+            raise ObjectNotFoundError()
