@@ -9,7 +9,7 @@ from rest_framework.request import Request
 from rest_framework.response import Response
 
 from . import _multipart, _registry
-from ._multipart import ObjectNotFoundException, TransferredPart, TransferredParts
+from ._multipart import ObjectNotFoundError, TransferredPart, TransferredParts
 
 
 class UploadInitializationRequestSerializer(serializers.Serializer):
@@ -17,6 +17,14 @@ class UploadInitializationRequestSerializer(serializers.Serializer):
     file_name = serializers.CharField(trim_whitespace=False)
     file_size = serializers.IntegerField(min_value=1)
     # part_size = serializers.IntegerField(min_value=1)
+    content_type = serializers.CharField(required=False)
+
+    def validate_field_id(self, field_id):
+        try:
+            _registry.get_field(field_id)
+        except KeyError:
+            raise serializers.ValidationError(f'Invalid field ID: "{field_id}".')
+        return field_id
 
 
 class PartInitializationResponseSerializer(serializers.Serializer):
@@ -78,13 +86,18 @@ def upload_initialize(request: Request) -> HttpResponseBase:
     upload_request: Dict = request_serializer.validated_data
     field = _registry.get_field(upload_request['field_id'])
 
+    file_name = upload_request['file_name']
     # TODO The first argument to generate_filename() is an instance of the model.
     # We do not and will never have an instance of the model during field upload.
     # Maybe we need a different generate method/upload_to with a different signature?
-    object_key = field.generate_filename(None, upload_request['file_name'])
+    object_key = field.generate_filename(None, file_name)
+
+    content_type = upload_request.get('content_type')
 
     initialization = _multipart.MultipartManager.from_storage(field.storage).initialize_upload(
-        object_key, upload_request['file_size']
+        object_key,
+        upload_request['file_size'],
+        content_type=content_type,
     )
 
     # signals.s3_file_field_upload_prepare.send(
@@ -160,7 +173,7 @@ def finalize(request: Request) -> HttpResponseBase:
     # We don't want to distribute the field value if the upload did not complete.
     try:
         size = _multipart.MultipartManager.from_storage(field.storage).get_object_size(object_key)
-    except ObjectNotFoundException:
+    except ObjectNotFoundError:
         return Response('Object not found', status=400)
 
     field_value = signing.dumps(
